package probetests.probetest1;// This class represents a polynomial. A polynomial is a sum of multiple monomials (class 'Monomial'),
// such as 3*x^1 + -1*x^2 + 1*x^5.
// 'Polynomial' uses a binary search tree to store its monomials. The degree of the monomial is the key.
// A specific degree exists at most once in the polynomial.

public class Polynomial {
    private TreeNode root;


    // Initializes this polynomial with multiple monomials. The coefficients of the monomials are
    // specified by 'coeffs', where coeffs[i] is the coefficient of the monomial of degree i.
    // Entries with value 0 are ignored, i.e. corresponding monomials are not stored in the polynomial.
    public Polynomial(int[] coeffs) {
        for (int i = 0; i < coeffs.length; i++) {
            if (coeffs[i] != 0) {
                add(coeffs[i], i);
            }
        }
    }

    // Adds the monomial specified by 'coeff' and 'degree' to this polynomial, if coeff != 0,
    // otherwise 'add' has no effect.
    // If this polynomial already has a monomial of the same degree, no new monomial is added, instead
    // the coefficient of the monomial is modified accordingly ('combine' is called).
    public void add(int coeff, int degree) {
        if (coeff != 0) {
            if (root == null) {
                root = new TreeNode(degree, new Monomial(coeff, degree));
                return;
            }
            TreeNode node = get(degree, root);
            if(degree == node.getKey()){
                node.getValue().combine(new Monomial(coeff, degree));
                return;
            }
            if (degree < node.getKey()) {
                node.setLeft(new TreeNode(degree, new Monomial(coeff, degree)));
            } else {
                node.setRight(new TreeNode(degree, new Monomial(coeff, degree)));
            }
        }
    }

    public TreeNode get(int degree, TreeNode node) {
        if (node.getKey() == degree)
            return node;
        if (degree < node.getKey()) {
            if (node.getLeft() == null)
                return node;
            return get(degree, node.getLeft());
        } else {
            if (node.getRight() == null)
                return node;
            return get(degree, node.getRight());
        }
    }

    // Adds all monomials of 'p' to this polynomial.
    // (The rules of 'add(int,int)' apply for each monomial to be added.)
    public void add(Polynomial p) {
        add(p.root);
    }

    public void add(TreeNode node) {
        if (node == null)
            return;
        add(node.getValue().getCoeff(), node.getValue().getDegree());
        add(node.getLeft());
        add(node.getRight());
    }

    // Returns the value of the polynomial for a specified value of 'x'
    public int eval(int x) {
        return eval(root, x);
    }

    public int eval(TreeNode node, int x) {
        if (node == null)
            return 0;
        int eval = node.getValue().eval(x);
        eval += eval(node.getLeft(), x);
        eval += eval(node.getRight(), x);
        return eval;
    }

    // Returns a polynomial representation in mathematical notation such as
    // "2*x^0 + 6*x^2 + -2*x^3", where monomials are ordered ascending according to
    // their degree. Note that a positive sign of the leftmost coefficient is
    // not shown.
    // Returns the string "0" if the polynomial has no monomials (is empty).
    public String toString() {
        return toString(root);
    }

    private String toString(TreeNode node) {
        String res = "";
        if(node.getLeft() != null){
            res += " + "+ toString(node.getLeft());
        }
        res+= " + "+node.getValue().toString();
        if(node.getRight() != null)
            res += " + "+toString(node.getRight());
        return res;
//        String res = node.getValue().toString();
//        if (node.getLeft() != null)
//            res += " + " + toString(node.getLeft());
//        if (node.getRight() != null)
//            res += " + " + toString(node.getRight());
//        return res;
    }

}

class TreeNode {

    private int key;
    private TreeNode left;
    private TreeNode right;
    private Monomial value;

    public TreeNode(int key, Monomial value) {
        this.key = key;
        this.value = value;
    }

    public int getKey() {
        return key;
    }

    public TreeNode getLeft() {
        return left;
    }

    public void setLeft(TreeNode left) {
        this.left = left;
    }

    public TreeNode getRight() {
        return right;
    }

    public void setRight(TreeNode right) {
        this.right = right;
    }

    public void setKey(int key) {
        this.key = key;
    }

    public Monomial getValue() {
        return value;
    }

    public void setValue(Monomial value) {
        this.value = value;
    }
}

