package probetests.probetest3;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Objects;

// The class implements 'Polynomial' and represents the form (a + x*p), where 'a' is an Integer
// and 'p' is another polynomial. 'p' must not be 'null' (objects of 'Constant' shall be used to represent constants).
// This class implements 'Polynomial' by using Horner's scheme.
// Example:
// a₀ + a₁*x + a₂*x² + a₃*x³ + a₄*x⁴
// The polynomial above is represented in Horner's scheme as:
// a₀ + x*(a₁ + x*(a₂ + x*(a₃ + x*a₄)))
//
public class HornerScheme implements Polynomial {

    private int number;
    private Polynomial polynomial;

    // Initializes this object.
    // Precondition: 'p' must not be 'null'.
    public HornerScheme(int a, Polynomial p) {
        this.number = a;
        this.polynomial = p;
    }

    // Creates a polynomial from the coefficients specified by the array coeffs = {a₀, a₁, a₂, ... a𝘥}.
    // Precondition: coeffs != null and coeffs has at least one element.
    public static Polynomial create(int[] coeffs) {
        Polynomial p = new Constant(coeffs[coeffs.length - 1]);
        for (int i = coeffs.length - 2; i >= 0; i--) {
            p = new HornerScheme(coeffs[i], p);
        }
        return p;
    }

    @Override
    public int degree() {
        List<Integer> list = coefficients();
        return list.size() - 1;
    }

    @Override
    public List<Integer> coefficients() {
        List<Integer> list = new ArrayList<>();
        for (int i : this) {
            list.add(i);
        }
        return list;
    }

    @Override
    public double eval(double x) {
        return (double) number + x * polynomial.eval(x);
    }

    @Override
    public Iterator<Integer> iterator() {
        return new Iterator<Integer>() {
            boolean hasNext = true;
            Iterator<Integer> iter = polynomial.iterator();

            @Override
            public boolean hasNext() {
                return hasNext || iter.hasNext();
            }

            @Override
            public Integer next() {
                if (hasNext) {
                    hasNext = false;
                    return number;
                } else {
                   return iter.next();
                }
            }
        };
    }

    @Override
    public int hashCode() {
        return Objects.hash(number, polynomial);
    }

    @Override
    public boolean equals(Object obj) {
        if(obj == null || obj.getClass() != getClass())
            return false;
        if(obj == this)
            return true;
        HornerScheme hs = (HornerScheme) obj;
        return hs.number == number && Objects.equals(hs.polynomial, polynomial);
    }

    @Override
    public String toString() {
        return String.format("(%d + x*%s)", number, polynomial);
    }
}



